// Copyright (c) 2011 Firebase.co and Contributors - http://www.firebase.co
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

var Expression = require('fire').Expression
var ValidationContext = require('./ValidationContext')
var ValidationError = require('./ValidationError')
var i18n = require('fire-i18n')
var CheckpointError = require('./CheckpointError')

function ExpressionBase() {
	
}

ExpressionBase.prototype = new Expression()
i18n.enableExpression(ExpressionBase)

ExpressionBase.prototype.getValidationContext = function() {
	this._validationContext = this._validationContext || this.getVar('validation')
	return this._validationContext
}

ExpressionBase.prototype.setValidationContext = function(context) {
	this.setScopeVar('validation', this._validationContext = context)
}

ExpressionBase.prototype.ensureValidationContext = function() {
	var currentContext = new ValidationContext()
	currentContext.target = this.getHintVariableValue()
	
	this.setValidationContext(currentContext)
}

ExpressionBase.prototype.getDefaultI18nErrorMessage = function(replacements) {
	var msg = this.getI18nText(this.expressionName, replacements)
	return msg
}

ExpressionBase.prototype.ensureContextOnMember = function(requireHint) {
	if(!this.hasHint()) {
		if(requireHint) {
			this.raiseError("Member validator '" + this.expressionName + "' requires a hint with the name of the member to validate")
			return false
		}
	}
	var memberName = this.hasHint() ? this.getHintValue() : null
	return this.setMemberContext(memberName)
}

ExpressionBase.prototype.checkValidationContext = function() {
	var currentContext = this.getValidationContext()
	if(!currentContext) {
		this.raiseError("Member validator '" + this.expressionName + "' requires a validation context")
		return false
	}
	return currentContext
}

ExpressionBase.prototype.setMemberContext = function(memberName) {
	if(memberName) {
		var currentContext = this.checkValidationContext()
		if(!currentContext) return
		
		var subContext = currentContext.createMemberContext(memberName)
		this.setValidationContext(subContext)
		return true
	}
	else {
		return true
	}
}

ExpressionBase.prototype.performCheckpoint = function() {
	var valContext = this.checkValidationContext()
	if(!valContext) return
	if(valContext.hasErrors()) {
		this.raiseError(new CheckpointError())
		return false
	} else {
		this.bypass()
		return true
	}
}

module.exports = ExpressionBase