// Copyright (c) 2011 Firebase.co and Contributors - http://www.firebase.co
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

function ValidationContext(path, parentContext) {
	this.path = path || '/'
	if(parentContext && !(parentContext instanceof ValidationContext)) {
		throw "parentContext must be instance of ValidationContext"
	}
	this.parentContext = parentContext || null
	this.target = null
	this.errors = []
}

ValidationContext.prototype.isRoot = function() {
	return this.path == '/'
}

ValidationContext.prototype.addError = function(error) {
	this.errors.push(error)
	if(!this.isRoot()) {
		this.parentContext.addError(error)
	}
}

ValidationContext.prototype.hasErrors = function() {
	return this.errors.length > 0
}

ValidationContext.prototype.createMemberContext = function(memberName) {
	var memberPath = this.path == '/' ? ("/" + memberName) : (this.path + "/" + memberName)
	var subContext = new ValidationContext(memberPath, this)
	if(this.target) {
		subContext.target = this.target[memberName]
	}
	return subContext
}

module.exports = ValidationContext